import logging
import time
import json
import urllib.parse
from configparser import ConfigParser
from pyasuswrt import AsusWrtHttp

logger = logging.getLogger(__name__)


class SpeedtestClient:
    def __init__(self, config: ConfigParser):
        self._config = config
        self._asuswrt_client = AsusWrtHttp(
            self._config.get('asus_router', 'host'),
            self._config.get('asus_router', 'username'),
            self._config.get('asus_router', 'password'),
            port=self._config.getint('asus_router', 'port'),
            use_https=self._config.getboolean('asus_router', 'use_https')
        )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        if self._asuswrt_client is not None:
            await self._asuswrt_client.async_disconnect()

    async def asus_get_speedtest_history(self):
        history = json.loads(await self._asuswrt_client._AsusWrtHttp__send_req('ookla_speedtest_get_history()'))
        return history['ookla_speedtest_get_history']

    def parse_speedtest_history(self, history: dict, limit: int):
        return history[:limit]

    async def asus_set_speedtest_start_time(self, start_time: int):
        data = f'ookla_start_time={start_time}'
        # data = {"ookla_start_time": start_time}
        await self._asuswrt_client._AsusWrtHttp__post(path='set_ookla_speedtest_start_time.cgi', command=data)

    async def asus_start_speedtest(self):
        data_type = ""
        data_id = ""
        data = f'type={data_type}&id={data_id}'
        # data = {"type": data_type, "id": data_id}
        await self._asuswrt_client._AsusWrtHttp__post(path='ookla_speedtest_exe.cgi', command=data)

    async def asus_get_speedtest_result(self):
        result = json.loads(await self._asuswrt_client._AsusWrtHttp__send_req('ookla_speedtest_get_result()'))
        return result['ookla_speedtest_get_result']

    async def wait_and_return_speedtest_result(self, timeout: int, poll_frequency: int):
        count = 0
        while (count <= timeout):
            time.sleep(poll_frequency)
            count += poll_frequency
            results = await self.asus_get_speedtest_result()
            if len(results) > 1:
                x = -1
                if len(results[x]) == 0:
                    x = -2
                latest_result = results[x]
                if latest_result['type'] == "result":
                    return latest_result
        raise Exception(f'Speedtest did not complete within {timeout} seconds')

    def convert_history_to_request_payload(self, history: dict):
        payload = ""
        for record in history:
            if len(record) > 0:
                payload += json.dumps(record, separators=(',', ':')) + '\n'
        return urllib.parse.quote_plus(payload)

    async def asus_write_speedtest_history(self, history: str):
        data = f'speedTest_history={history}'
        await self._asuswrt_client._AsusWrtHttp__post(path='ookla_speedtest_write_history.cgi', command=data)

    async def save_speedtest_results(self, result: dict, history_limit: int):
        history = self.parse_speedtest_history(await self.asus_get_speedtest_history(), history_limit)
        history.insert(0, result)
        payload = self.convert_history_to_request_payload(history)

        error = None
        success = None
        try:
            await self.asus_write_speedtest_history(payload)
            success = True
        except Exception as e:
            success = False
            error = e
        finally:
            return {
                'success': success,
                'data': payload,
                'error': error
            }

    async def run(self):
        timestamp = int(time.time())

        print('Setting speedtest start time')
        await self.asus_set_speedtest_start_time(timestamp)
        print('Setting speedtest start time...complete')

        print(f'Starting speedtest at {timestamp}')
        await self.asus_start_speedtest()

        print('Waiting for speedtest to finish')
        speedtest_result = await self.wait_and_return_speedtest_result(
            timeout=self._config.getint('speedtest', 'timeout'),
            poll_frequency=self._config.getint('speedtest', 'poll_frequency')
        )
        print('Waiting for speedtest to finish...complete')

        print('Saving speedtest result')
        save_result = await self.save_speedtest_results(
            result=speedtest_result,
            history_limit=self._config.getint('speedtest', 'history_limit')
        )
        print('Saving speedtest result...complete')
        print(f'Save successful: {save_result['success']}')



import pytest
import configparser
import json
from unittest.mock import patch
from pyasuswrt import AsusWrtHttp
from src.asuswrtspeedtest import SpeedtestClient


@pytest.fixture(scope="module")
def speedtest_client():
    config = configparser.ConfigParser()
    config.read('tests/config.ini')
    return SpeedtestClient(config)


@pytest.fixture()
def mock_get_speedtest_history():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__send_req') as m:
        with open('tests/fixtures/ookla_speedtest_get_history.json', 'r') as file:
            m.return_value = file.read()
        yield m


@pytest.fixture()
def mock_latest_speedtest_result():
    with open('tests/fixtures/ookla_speedtest_latest_result.json', 'r') as file:
        yield json.load(file)


@pytest.fixture()
def mock_speedtest_history_payload():
    with open('tests/fixtures/ookla_speedtest_history_payload.txt', 'r') as file:
        yield file.read().strip()


@pytest.fixture()
def mock_speedtest_updated_history_payload():
    with open('tests/fixtures/ookla_speedtest_updated_history_payload.txt', 'r') as file:
        yield file.read().strip()


@pytest.fixture()
def mock_write_speedtest_result():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__post') as m:
        yield m


@pytest.fixture()
def mock_write_speedtest_result_failure():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__post') as m:
        m.side_effect = Exception('request failed')
        yield m


@pytest.mark.asyncio
async def test_convert_history_to_request_payload(
    speedtest_client,
    mock_get_speedtest_history,
    mock_speedtest_history_payload
):
    data = await speedtest_client.asus_get_speedtest_history()
    payload = speedtest_client.convert_history_to_request_payload(data)
    assert isinstance(payload, str)
    assert payload == mock_speedtest_history_payload


@pytest.mark.asyncio
async def test_save_speedtest_results(
    speedtest_client,
    mock_get_speedtest_history,
    mock_write_speedtest_result,
    mock_latest_speedtest_result,
    mock_speedtest_updated_history_payload
):
    history_limit = 10
    result = await speedtest_client.save_speedtest_results(mock_latest_speedtest_result, history_limit)
    assert isinstance(result, dict)
    assert result['success'] == True
    assert result['error'] is None
    assert result['data'] == mock_speedtest_updated_history_payload


@pytest.mark.asyncio
async def test_save_speedtest_results_failure(
    speedtest_client,
    mock_get_speedtest_history,
    mock_write_speedtest_result_failure,
    mock_latest_speedtest_result,
    mock_speedtest_updated_history_payload
):
    history_limit = 10
    result = await speedtest_client.save_speedtest_results(mock_latest_speedtest_result, history_limit)
    assert isinstance(result, dict)
    assert result['success'] == False
    assert result['error'] is not None
    assert result['data'] is not None



import asyncio
import logging
import sys
import configparser
from src.asuswrtspeedtest import SpeedtestClient

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
logging.getLogger('asyncio').setLevel(logging.WARNING)
logger = logging.getLogger(__name__)

config = configparser.ConfigParser()
config.read('config/config.ini')


async def run_speedtest():
    async with SpeedtestClient(config) as speedtest_client:
        await speedtest_client.run()

asyncio.run(run_speedtest())



import pytest
import configparser
from unittest.mock import patch
from pyasuswrt import AsusWrtHttp
from src.asuswrtspeedtest import SpeedtestClient


@pytest.fixture(scope="module")
def speedtest_client():
    config = configparser.ConfigParser()
    config.read('tests/config.ini')
    return SpeedtestClient(config)


@pytest.fixture()
def mock_get_speedtest_history():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__send_req') as m:
        with open('tests/fixtures/ookla_speedtest_get_history.json', 'r') as file:
            m.return_value = file.read()
        yield m


@pytest.fixture()
def mock_get_speedtest_result():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__send_req') as m:
        with open('tests/fixtures/ookla_speedtest_get_result.json', 'r') as file:
            m.return_value = file.read()
        yield m


@pytest.fixture()
def mock_get_incomplete_speedtest_result():
    with patch.object(AsusWrtHttp, '_AsusWrtHttp__send_req') as m:
        with open('tests/fixtures/ookla_speedtest_get_incomplete_result.json', 'r') as file:
            m.return_value = file.read()
        yield m


@pytest.mark.asyncio
async def test_get_speedtest_history(speedtest_client, mock_get_speedtest_history):
    data = await speedtest_client.asus_get_speedtest_history()
    assert isinstance(data, list)
    assert len(data) == 19


@pytest.mark.asyncio
async def test_parse_speedtest_history(speedtest_client, mock_get_speedtest_history):
    data = await speedtest_client.asus_get_speedtest_history()
    history_limit = 5
    parsed_data = speedtest_client.parse_speedtest_history(history=data, limit=history_limit)
    assert len(parsed_data) == history_limit


@pytest.mark.asyncio
async def test_get_speedtest_result(speedtest_client, mock_get_speedtest_result):
    data = await speedtest_client.asus_get_speedtest_result()
    assert isinstance(data, list)
    last_result = data[-2]
    assert last_result['type'] == 'result'


@pytest.mark.asyncio
async def test_wait_and_return_speedtest_result(speedtest_client, mock_get_speedtest_result):
    data = await speedtest_client.wait_and_return_speedtest_result(timeout=3, poll_frequency=1)
    assert isinstance(data, dict)
    assert data['type'] == 'result'
    assert data['result']['persisted'] == True


@pytest.mark.asyncio
async def test_wait_and_return_on_speedtest_result_timeout(speedtest_client, mock_get_incomplete_speedtest_result):
    timeout = 2
    poll_frequency = 1
    with pytest.raises(Exception) as excinfo:
        await speedtest_client.wait_and_return_speedtest_result(timeout=timeout, poll_frequency=poll_frequency)
    assert str(excinfo.value) == f'Speedtest did not complete within {timeout} seconds'
